@@ -11,12 +11,18 @@
 #' If provided, comparisons are performed within each block to ensure that block effects do not confound the estimates.
 #' The weighted average of the effect sizes across all blocks is reported for each gene.
 #' Alternatively \code{NULL}, if all cells are from the same block.
+#' @param block.average.policy String specifying the policy to use for average statistics across blocks.
+#' This can either be a (weighted) \code{"mean"} or a \code{"quantile"}.
+#' Only used if \code{block} is not \code{NULL}. 
 #' @param block.weight.policy String specifying the policy to use for weighting different blocks when computing the average for each statistic.
 #' See the argument of the same name in \code{\link{computeBlockWeights}} for more detail.
 #' Only used if \code{block} is not \code{NULL}.
 #' @param variable.block.weight Numeric vector of length 2, specifying the parameters for variable block weighting.
 #' See the argument of the same name in \code{\link{computeBlockWeights}} for more detail.
 #' Only used if \code{block} is not \code{NULL} and \code{block.weight.policy = "variable"}.
+#' @param block.quantile Number specifying the probability of the quantile of statistics across blocks. 
+#' Defaults to 0.5, i.e., the median of per-block statistics.
+#' Only used if \code{block} is not \code{NULL} and \code{block.average.policy="quantile"}.
 #' @param threshold Non-negative numeric scalar specifying the minimum threshold on the differences in means (i.e., the log-fold change, if \code{x} contains log-expression values). 
 #' This is incorporated into the effect sizes for Cohen's d and the AUC.
 #' Larger thresholds will favor genes with large differences at the expense of genes with low variance that would otherwise have comparable effect sizes.
@@ -27,6 +33,20 @@
 #' @param compute.cohens.d Logical scalar indicating whether to compute Cohen's d.
 #' @param compute.auc Logical scalar indicating whether to compute the AUC.
 #' Setting this to \code{FALSE} can improve speed and memory efficiency.
+#' @param compute.summary.min Boolean specifying whether to compute the minimum as a summary statistic for each effect size.
+#' Only used if \code{all.pairwise=FALSE}.
+#' @param compute.summary.mean Boolean specifying whether to compute the mean as a summary statistic for each effect size.
+#' Only used if \code{all.pairwise=FALSE}.
+#' @param compute.summary.median Boolean specifying whether to compute the median as a summary statistic for each effect size.
+#' Only used if \code{all.pairwise=FALSE}.
+#' @param compute.summary.max Boolean specifying whether to compute the maximum as a summary statistic for each effect size.
+#' Only used if \code{all.pairwise=FALSE}.
+#' @param compute.summary.quantiles Numeric scalars containing the probabilities of quantiles to compute as summary statistics for each effect size.
+#' If \code{NULL}, no quantiles are computed.
+#' Only used if \code{all.pairwise=FALSE}.
+#' @param compute.summary.min.rank Boolean specifying whether to compute the mininum rank as a summary statistic for each effect size.
+#' If \code{NULL}, no quantiles are computed.
+#' Only used if \code{all.pairwise=FALSE}.
 #' @param all.pairwise Logical scalar indicating whether to report the effect sizes for every pairwise comparison between groups.
 #' Alternatively, an integer scalar indicating the number of top markers to report from each pairwise comparison between groups.
 #' If \code{FALSE}, only the summary statistics are reported.
@@ -39,7 +59,7 @@
 #' \itemize{
 #' \item \code{cohens.d}, a list of data frames where each data frame corresponds to a group.
 #' Each row of each data frame represents a gene, while each column contains a summary of Cohen's d from pairwise comparisons to all other groups.
-#' This includes the \code{min}, \code{mean}, \code{median}, \code{max} and \code{min.rank} - check out \code{?\link{summarizeEffects}} for details.
+#' This includes \code{min}, \code{mean}, \code{median}, \code{max}, \code{quantile.*} and \code{min.rank} - check out \code{?\link{summarizeEffects}} for details.
 #' Omitted if \code{compute.cohens.d=FALSE}.
 #' \item \code{auc}, a list like \code{cohens.d} but containing the summaries of the AUCs from each pairwise comparison.
 #' Omitted if \code{compute.auc=FALSE}.
@@ -155,20 +175,29 @@ scoreMarkers <- function(
     x, 
     groups, 
     block=NULL, 
+    block.average.policy=c("mean", "quantile"),
     block.weight.policy=c("variable", "equal", "none"),
     variable.block.weight=c(0, 1000),
+    block.quantile=0.5,
     compute.group.mean=TRUE,
     compute.group.detected=TRUE,
     compute.delta.mean=TRUE,
     compute.delta.detected=TRUE,
     compute.cohens.d=TRUE,
     compute.auc=TRUE,
+    compute.summary.min=TRUE,
+    compute.summary.mean=TRUE,
+    compute.summary.median=TRUE,
+    compute.summary.max=TRUE,
+    compute.summary.quantiles=NULL,
+    compute.summary.min.rank=TRUE,
     threshold=0, 
     all.pairwise=FALSE, 
     min.rank.limit=500,
     num.threads=1
 ) {
     rn <- rownames(x)
+    ngenes <- nrow(x)
     x <- initializeCpp(x, .check.na=FALSE)
     groups <- .transformFactor(groups)
     block <- .transformFactor(block)
@@ -177,8 +206,10 @@ scoreMarkers <- function(
         groups=groups$index,
         num_groups=length(groups$names),
         block=block$index,
+        block_average_policy=match.arg(block.average.policy),
         block_weight_policy=match.arg(block.weight.policy),
         variable_block_weight=variable.block.weight,
+        block_quantile=block.quantile,
         threshold=threshold,
         compute_group_mean=compute.group.mean,
         compute_group_detected=compute.group.detected,
@@ -189,35 +220,45 @@ scoreMarkers <- function(
         num_threads=num.threads
     )
 
-    keep.effects <- character(0)
-    if (compute.cohens.d) {
-        keep.effects <- c(keep.effects, "cohens.d")
-    }
-    if (compute.auc) {
-        keep.effects <- c(keep.effects, "auc")
-    }
-    if (compute.delta.mean) {
-        keep.effects <- c(keep.effects, "delta.mean")
-    }
-    if (compute.delta.detected) {
-        keep.effects <- c(keep.effects, "delta.detected")
-    }
-
     if (isTRUE(all.pairwise)) {
         output <- do.call(score_markers_pairwise, c(list(x), args))
-        for (nm in keep.effects) {
+        for (nm in setdiff(names(output), c("mean", "detected"))) {
             current <- output[[nm]]
             dimnames(current) <- list(groups$names, groups$names, rn)
             output[[nm]] <- current
         }
 
     } else if (isFALSE(all.pairwise)) {
-        output <- do.call(score_markers_summary, c(list(x, min_rank_limit=min.rank.limit), args))
-        for (nm in keep.effects) {
+        if (!is.null(compute.summary.quantiles)) {
+            compute.summary.quantiles <- sort(unique(compute.summary.quantiles))
+        }
+
+        output <- do.call(
+            score_markers_summary,
+            c(
+                list(
+                    x,
+                    min_rank_limit=min.rank.limit,
+                    compute_summary_mean=compute.summary.mean,
+                    compute_summary_min=compute.summary.min,
+                    compute_summary_median=compute.summary.median,
+                    compute_summary_max=compute.summary.max,
+                    compute_summary_quantiles=compute.summary.quantiles,
+                    compute_summary_min_rank=compute.summary.min.rank
+                ),
+                args
+            )
+        )
+
+        for (nm in setdiff(names(output), c("mean", "detected"))) {
             current <- output[[nm]]
             names(current) <- groups$names
             for (i in seq_along(current)) {
-                df <- data.frame(current[[i]])
+                if (length(current[[i]])) {
+                    df <- data.frame(current[[i]])
+                } else {
+                    df <- data.frame(matrix(0, ngenes, 0))
+                }
                 rownames(df) <- rn
                 current[[i]] <- df
             }
@@ -226,7 +267,7 @@ scoreMarkers <- function(
 
     } else {
         output <- do.call(score_markers_best, c(list(x, top=all.pairwise), args))
-        for (nm in keep.effects) {
+        for (nm in setdiff(names(output), c("mean", "detected"))) {
             current <- output[[nm]]
             names(current) <- groups$names
             for (i in seq_along(current)) {
@@ -236,17 +277,14 @@ scoreMarkers <- function(
         }
     }
 
-    keep.stats <- character()
     if (compute.group.mean) {
         dimnames(output$mean) <- list(rn, groups$names)
-        keep.stats <- c(keep.stats, "mean")
     }
     if (compute.group.detected) {
         dimnames(output$detected) <- list(rn, groups$names)
-        keep.stats <- c(keep.stats, "detected")
     }
 
-    output[c(keep.stats, keep.effects)]
+    output
 }
 
 #' Report marker statistics for a single group
